@@ -6,10 +6,25 @@
 
 #define INTEGRITY_CHECKS 1
 /* #define DEBUG_ALLOCS 1 */
-/*#define PGIS_DEBUG 1
-#define PGIS_DEBUG_CALLS 1*/
+/* #define PGIS_DEBUG 1
+#define PGIS_DEBUG_CALLS 1 */
+/*#define PGIS_DEBUG_ALLOCS 1 */
 /* #define DEBUG_CALLS 1 */
 
+/*
+ * Floating point comparitors.
+ */
+#define PGIS_EPSILON 1e-12
+#define FP_MAX(A, B) ((A > B) ? A : B)
+#define FP_MIN(A, B) ((A < B) ? A : B)
+#define FP_LT(A, B) ((A + PGIS_EPSILON) < B)
+#define FP_LTEQ(A, B) ((A - PGIS_EPSILON) <= B)
+#define FP_CONTAINS_TOP(A, X, B) (FP_LT(A, X) && FP_LTEQ(X, B))
+#define FP_CONTAINS_BOTTOM(A, X, B) (FP_LTEQ(A, X) && FP_LT(X, B))
+#define FP_CONTAINS_INCL(A, X, B) (FP_LTEQ(A, X) && FP_LTEQ(X, B))
+#define FP_CONTAINS_EXCL(A, X, B) (FP_LT(A, X) && FP_LT(X, B))
+#define FP_CONTAINS(A, X, B) FP_CONTAINS_EXCL(A, X, B)
+
 /*
  * Memory management function types
  */
@@ -472,9 +487,9 @@ extern char lwgeom_hasBBOX(uchar type); /* true iff B bit set    */
  * with postgresql varlena struct
  */
 typedef struct {
-	uint32 size;
-	uchar type; /* encodes ndims, type, bbox presence,
-		       srid presence */
+	uint32 size;        /* varlena header (do not touch directly!) */
+	uchar type;         /* encodes ndims, type, bbox presence,
+		                srid presence */
 	uchar data[1];
 } PG_LWGEOM;
 
@@ -511,9 +526,19 @@ char is_worth_caching_lwgeom_bbox(const LWGEOM *);
  * by most functions from an PG_LWGEOM struct.
  * (which is an PG_LWGEOM w/out int32 size casted to char *)
  */
-/*#define SERIALIZED_FORM(x) ((uchar *)(x))+4*/
 #define SERIALIZED_FORM(x) ((uchar *)VARDATA((x)))
 
+/*
+    This structure is a "glue" structure for returning a serialized
+    LWGEOM from the parser, along with its size. By using a separate
+    type, we remove the constraint that the output from the
+    parser must be PG_LWGEOM format (and hence protect ourselves
+    from future varlena changes)
+*/
+typedef struct serialized_lwgeom {
+    uchar *lwgeom;
+    int size;
+} SERIALIZED_LWGEOM;
 
 /*
  * This function computes the size in bytes
@@ -863,8 +888,8 @@ extern void pfree_POINTARRAY(POINTARRAY *pa);
  * utility
  ****************************************************************/
 
-extern uint32 get_uint32(const uchar *loc);
-extern int32 get_int32(const uchar *loc);
+extern uint32 lw_get_uint32(const uchar *loc);
+extern int32 lw_get_int32(const uchar *loc);
 extern void printBOX3D(BOX3D *b);
 extern void printPA(POINTARRAY *pa);
 extern void printLWPOINT(LWPOINT *point);
@@ -1056,7 +1081,7 @@ extern LWCOLLECTION *lwcollection_segmentize2d(LWCOLLECTION *coll, double dist);
 
 extern uchar parse_hex(char *str);
 extern void deparse_hex(uchar str, char *result);
-extern uchar *parse_lwgeom_wkt(char *wkt_input);
+extern SERIALIZED_LWGEOM *parse_lwgeom_wkt(char *wkt_input);
 
 extern char *lwgeom_to_ewkt(LWGEOM *lwgeom);
 extern char *lwgeom_to_hexwkb(LWGEOM *lwgeom, unsigned int byteorder);
@@ -1077,7 +1102,6 @@ extern char getMachineEndian(void);
 
 void errorIfSRIDMismatch(int srid1, int srid2);
 
-
 /* CURVETYPE */
 typedef struct
 {
@@ -1137,6 +1161,8 @@ typedef struct
  * LWCURVE functions
  ******************************************************************/
 
+LWCURVE *lwcurve_construct(int SRID, BOX2DFLOAT4 *bbox, POINTARRAY *points);
+
 /*
  * given the LWGEOM serialized form (or a pointer into a muli* one)
  * construct a proper LWCURVE.
@@ -1167,6 +1193,7 @@ LWGEOM *lwcurve_add(const LWCURVE *to, uint32 where, const LWGEOM *what);
 extern int lwcurve_compute_box2d_p(LWCURVE *curve, BOX2DFLOAT4 *box);
 extern BOX3D *lwcurve_compute_box3d(LWCURVE *curve);
 extern void pfree_curve(LWCURVE  *curve);
+LWCURVE *lwcurve_clone(const LWCURVE *curve);
 
 /******************************************************************
  * LWMULTIx and LWCOLLECTION functions
@@ -1187,6 +1214,16 @@ LWGEOM *lwmcurve_add(const LWMCURVE *to, uint32 where, const LWGEOM *what);
 LWMSURFACE *lwmsurface_deserialize(uchar *serialized_form);
 
 LWGEOM *lwmsurface_add(const LWMSURFACE *to, uint32 where, const LWGEOM *what);
+
+/*******************************************************************************
+ * SQLMM internal functions
+ ******************************************************************************/
+
+uint32 has_arc(LWGEOM *geom);
+double lwcircle_center(POINT4D *p1, POINT4D *p2, POINT4D *p3, POINT4D **result);
+LWGEOM *lwgeom_segmentize(LWGEOM *geom, uint32 perQuad);
+extern double lwgeom_curvepolygon_area(LWCURVEPOLY *curvepoly);
+double lwcircle_center(POINT4D *p1, POINT4D *p2, POINT4D *p3, POINT4D **result);
+
 #endif /* !defined _LIBLWGEOM_H  */
 
-double lwcircle_center(POINT4D *p1, POINT4D *p2, POINT4D *p3, POINT4D **result);
