@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwcollection.c 2532 2006-12-01 22:16:44Z mleslie $
+ * $Id: lwcollection.c 2633 2007-06-28 20:16:13Z mleslie $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -31,7 +31,7 @@ lwcollection_construct(unsigned int type, int SRID, BOX2DFLOAT4 *bbox,
 #endif
 
 #ifdef PGIS_DEBUG_CALLS
-        lwnotice("lwcollection_construct called.");
+        lwnotice("lwcollection_construct called with %d, %d, %p, %d, %p.", type, SRID, bbox, ngeoms, geoms);
 #endif        
 
 	hasz = 0;
