@@ -22,7 +22,7 @@
 /*#define PGIS_DEBUG 1 */
 
 #ifndef MAXFLOAT
-  #define MAXFLOAT      3.40282347e+38F
+  #define MAXFLOAT      3.402823466e+38F
 #endif
 
 /*
@@ -32,7 +32,9 @@
 LWCURVE *
 lwcurve_construct(int SRID, BOX2DFLOAT4 *bbox, POINTARRAY *points)
 {
-        /*
+        LWCURVE *result;
+        
+	/*
          * The first arc requires three points.  Each additional
          * arc requires two more points.  Thus the minimum point count
          * is three, and the count must be odd.
@@ -43,7 +45,6 @@ lwcurve_construct(int SRID, BOX2DFLOAT4 *bbox, POINTARRAY *points)
                 return NULL;
         }
         
-        LWCURVE *result;
         result = (LWCURVE*) lwalloc(sizeof(LWCURVE));
 
         result->type = lwgeom_makeType_full(
@@ -106,7 +107,7 @@ lwcurve_deserialize(uchar *serialized_form)
 #ifdef PGIS_DEBUG
                 lwnotice("lwcurve_deserialize: input has srid");
 #endif
-                result->SRID = get_int32(loc);               
+                result->SRID = lw_get_int32(loc);               
                 loc += 4; /* type + SRID */
          }
          else
@@ -119,7 +120,7 @@ lwcurve_deserialize(uchar *serialized_form)
 
          /* we've read the type (1 byte) and SRID (4 bytes, if present) */
 
-         npoints = get_uint32(loc);
+         npoints = lw_get_uint32(loc);
 #ifdef PGIS_DEBUG
          lwnotice("curve npoints = %d", npoints);
 #endif         
@@ -504,7 +505,7 @@ lwgeom_size_curve(const uchar *serialized_curve)
         }
 
         /* we've read the type (1 byte) and SRID (4 bytes, if present) */
-        npoints = get_uint32(loc);
+        npoints = lw_get_uint32(loc);
         result += sizeof(uint32); /* npoints */
 
         result += TYPE_NDIMS(type) * sizeof(double) * npoints;
