@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom.c 2532 2006-12-01 22:16:44Z mleslie $
+ * $Id: lwgeom.c 2687 2007-08-13 07:06:58Z mcayland $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -220,6 +220,9 @@ lwgeom_reverse(LWGEOM *lwgeom)
 int
 lwgeom_compute_box2d_p(LWGEOM *lwgeom, BOX2DFLOAT4 *buf)
 {
+#ifdef PGIS_DEBUG_CALLS
+        lwnotice("lwgeom_compute_box2d_p called of %p of type %d.", lwgeom, TYPE_GETTYPE(lwgeom->type));
+#endif
 	switch(TYPE_GETTYPE(lwgeom->type))
 	{
 		case POINTTYPE:
@@ -373,17 +376,26 @@ lwgeom_release(LWGEOM *lwgeom)
 LWGEOM *
 lwgeom_clone(const LWGEOM *lwgeom)
 {
+#ifdef PGIS_DEBUG_CALLS
+        lwnotice("lwgeom_clone called with %p, %d", lwgeom, TYPE_GETTYPE(lwgeom->type));
+#endif
 	switch(TYPE_GETTYPE(lwgeom->type))
 	{
 		case POINTTYPE:
 			return (LWGEOM *)lwpoint_clone((LWPOINT *)lwgeom);
 		case LINETYPE:
 			return (LWGEOM *)lwline_clone((LWLINE *)lwgeom);
+                case CURVETYPE:
+                        return (LWGEOM *)lwcurve_clone((LWCURVE *)lwgeom);
 		case POLYGONTYPE:
 			return (LWGEOM *)lwpoly_clone((LWPOLY *)lwgeom);
+                case COMPOUNDTYPE:
+                case CURVEPOLYTYPE:
 		case MULTIPOINTTYPE:
 		case MULTILINETYPE:
+                case MULTICURVETYPE:
 		case MULTIPOLYGONTYPE:
+                case MULTISURFACETYPE:
 		case COLLECTIONTYPE:
 			return (LWGEOM *)lwcollection_clone((LWCOLLECTION *)lwgeom);
 		default:
@@ -525,22 +537,22 @@ lwgeom_from_ewkb(uchar *ewkb, size_t size)
 	size_t hexewkblen = size*2;
 	char *hexewkb;
 	long int i;
-	uchar *pglwgeom; /* This is a PG_LWGEOM */
 	LWGEOM *ret;
+    SERIALIZED_LWGEOM *serialized_lwgeom;
 
 	/* "HEXify" the EWKB */
 	hexewkb = lwalloc(hexewkblen+1);
 	for (i=0; i<size; ++i) deparse_hex(ewkb[i], &hexewkb[i*2]);
 	hexewkb[hexewkblen] = '\0';
 
-	/* Rely on grammar parser to construct a PG_LWGEOM */
-	pglwgeom = parse_lwgeom_wkt(hexewkb);
+	/* Rely on grammar parser to construct a LWGEOM */
+    serialized_lwgeom = parse_lwgeom_wkt(hexewkb);
 
 	/* Free intermediate HEXified representation */
 	lwfree(hexewkb);
 
 	/* Deserialize */
-	ret = lwgeom_deserialize(pglwgeom+4);
+	ret = lwgeom_deserialize(serialized_lwgeom->lwgeom);
 
 	return ret;
 }
