@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_functions_analytic.c 2580 2007-01-11 01:40:56Z mleslie $
+ * $Id: lwgeom_functions_analytic.c 2677 2007-08-02 19:57:38Z mleslie $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -15,6 +15,8 @@
 #include "liblwgeom.h"
 #include "lwgeom_pg.h"
 #include "math.h"
+#include "lwgeom_rtree.h"
+
 
 /***********************************************************************
  * Simple Douglas-Peucker line simplification. 
@@ -1073,25 +1075,95 @@ double determineSide(POINT2D *seg1, POINT2D *seg2, POINT2D *point)
 }
 
 /*
- * return 0 iff point is outside ring pts
+ * This function doesn't test that the point falls on the line defined by 
+ * the two points.  It assumes that that has already been determined
+ * by having determineSide return within the tolerance.  It simply checks
+ * that if the point is on the line, it is within the endpoints.
+ *
+ * returns: 1 if the point is not outside the bounds of the segment
+ *          0 if it is 
  */
-int point_in_ring(POINTARRAY *pts, POINT2D *point)
+int isOnSegment(POINT2D *seg1, POINT2D *seg2, POINT2D *point)
+{
+        double maxX; 
+        double maxY;
+        double minX;
+        double minY;
+
+        if(seg1->x > seg2->x)
+        {
+                maxX = seg1->x;
+                minX = seg2->x;
+        }
+        else
+        {
+                maxX = seg2->x;
+                minX = seg1->x;
+        }
+        if(seg1->y > seg2->y)
+        {
+                maxY = seg1->y;
+                minY = seg2->y;
+        }
+        else
+        {
+                maxY = seg2->y;
+                minY = seg1->y;
+        }
+#ifdef PGIS_DEBUG
+        lwnotice("maxX minX/maxY minY: %.8f %.8f/%.8f %.8f", maxX, minX, maxY, minY);
+#endif
+
+        if(maxX < point->x || minX > point->x)
+        {
+#ifdef PGIS_DEBUG
+                lwnotice("X value %.8f falls outside the range %.8f-%.8f", point->x, minX, maxX);
+#endif
+                return 0;
+        }
+        else if(maxY < point->y || minY > point->y)
+        {
+#ifdef PGIS_DEBUG
+                lwnotice("Y value %.8f falls outside the range %.8f-%.8f", point->y, minY, maxY);
+#endif
+                return 0;
+        }
+        return 1;
+}
+
+/*
+ * return -1 iff point is outside ring pts
+ * return 1 iff point is inside ring pts
+ * return 0 iff point is on ring pts
+ */
+int point_in_ring(RTREE_NODE *root, POINT2D *point)
 {
         int wn = 0;
         int i;
         double side;
         POINT2D seg1;
         POINT2D seg2;
-
+        LWMLINE *lines;
 #ifdef PGIS_DEBUG_CALLS
         lwnotice("point_in_ring called.");
 #endif
 
-        for(i=0; i<pts->npoints-1; i++)
+        lines = findLineSegments(root, point->y);
+        if(!lines)
+                return -1;
+
+        for(i=0; i<lines->ngeoms; i++)
         {
-                getPoint2d_p(pts, i, &seg1);
-                getPoint2d_p(pts, i+1, &seg2);
+                getPoint2d_p(lines->geoms[i]->points, 0, &seg1);
+                getPoint2d_p(lines->geoms[i]->points, 1, &seg2);
+
+
                 side = determineSide(&seg1, &seg2, point);
+#ifdef PGIS_DEBUG
+                lwnotice("segment: (%.8f, %.8f),(%.8f, %.8f)", seg1.x, seg1.y, seg2.x, seg2.y);
+                lwnotice("side result: %.8f", side);
+                lwnotice("counterclockwise wrap %d, clockwise wrap %d", FP_CONTAINS_BOTTOM(seg1.y,point->y,seg2.y), FP_CONTAINS_BOTTOM(seg2.y,point->y,seg1.y));
+#endif
                 /* zero length segments are ignored. */
                 if(((seg2.x-seg1.x)*(seg2.x-seg1.x)+(seg2.y-seg1.y)*(seg2.y-seg1.y)) < 1e-12*1e-12) 
                 {
@@ -1104,40 +1176,149 @@ int point_in_ring(POINTARRAY *pts, POINT2D *point)
                 /* a point on the boundary of a ring is not contained. */
                 if(fabs(side) < 1e-12) 
                 {
+                        if(isOnSegment(&seg1, &seg2, point) == 1)
+                        {
 #ifdef PGIS_DEBUG
-                        lwnotice("point on ring boundary between points %d, %d", i, i+1);
+                                lwnotice("point on ring boundary between points %d, %d", i, i+1);
 #endif
-                        return 0;
+                                return 0;
+                        }
                 }
-                else if(seg1.y < point->y && seg2.y > point->y && side > 0)
-                        ++wn;
-                else if(seg1.y > point->y && seg2.y < point->y && side < 0)
-                        --wn;
-        }
+                /*
+                 * If the point is to the left of the line, and it's rising,
+                 * then the line is to the right of the point and 
+                 * circling counter-clockwise, so incremement.
+                 */
+                else if(FP_CONTAINS_BOTTOM(seg1.y,point->y,seg2.y) && side>0)
+                {
 #ifdef PGIS_DEBUG
-        lwnotice("returning %d", wn);
+                        lwnotice("incrementing winding number.");
 #endif
-        return wn;
+                        ++wn;
+                }
+                /*
+                 * If the point is to the right of the line, and it's falling,
+                 * then the line is to the right of the point and circling
+                 * clockwise, so decrement.
+                 */
+                else if(FP_CONTAINS_BOTTOM(seg2.y,point->y,seg1.y) && side<0)
+                {
+#ifdef PGIS_DEBUG
+                        lwnotice("decrementing winding number.");
+#endif
+                        --wn;
+                }
+        }
+
+#ifdef PGIS_DEBUG
+        lwnotice("winding number %d", wn);
+#endif
+        if(wn == 0)
+        	return -1;
+        return 1;
+}
+
+
+/*
+ * return -1 iff point is outside ring pts
+ * return 1 iff point is inside ring pts
+ * return 0 iff point is on ring pts
+ */
+int point_in_ring_deprecated(POINTARRAY *pts, POINT2D *point)
+{
+        int wn = 0;
+        int i;
+        double side;
+        POINT2D seg1;
+        POINT2D seg2;
+#ifdef PGIS_DEBUG_CALLS
+        lwnotice("point_in_ring called.");
+#endif
+
+        
+        for(i=0; i<pts->npoints-1; i++)
+        {
+                getPoint2d_p(pts, i, &seg1);
+                getPoint2d_p(pts, i+1, &seg2);
+
+
+                side = determineSide(&seg1, &seg2, point);
+#ifdef PGIS_DEBUG
+                lwnotice("segment: (%.8f, %.8f),(%.8f, %.8f)", seg1.x, seg1.y, seg2.x, seg2.y);
+                lwnotice("side result: %.8f", side);
+                lwnotice("counterclockwise wrap %d, clockwise wrap %d", FP_CONTAINS_BOTTOM(seg1.y,point->y,seg2.y), FP_CONTAINS_BOTTOM(seg2.y,point->y,seg1.y));
+#endif
+                /* zero length segments are ignored. */
+                if(((seg2.x-seg1.x)*(seg2.x-seg1.x)+(seg2.y-seg1.y)*(seg2.y-seg1.y)) < 1e-12*1e-12) 
+                {
+#ifdef PGIS_DEBUG
+                        lwnotice("segment is zero length... ignoring.");
+#endif
+                        continue;
+                }
+
+                /* a point on the boundary of a ring is not contained. */
+                if(fabs(side) < 1e-12) 
+                {
+                        if(isOnSegment(&seg1, &seg2, point) == 1)
+                        {
+#ifdef PGIS_DEBUG
+                                lwnotice("point on ring boundary between points %d, %d", i, i+1);
+#endif
+                                return 0;
+                        }
+                }
+                /*
+                 * If the point is to the left of the line, and it's rising,
+                 * then the line is to the right of the point and 
+                 * circling counter-clockwise, so incremement.
+                 */
+                else if(FP_CONTAINS_BOTTOM(seg1.y,point->y,seg2.y) && side>0)
+                {
+#ifdef PGIS_DEBUG
+                        lwnotice("incrementing winding number.");
+#endif
+                        ++wn;
+                }
+                /*
+                 * If the point is to the right of the line, and it's falling,
+                 * then the line is to the right of the point and circling
+                 * clockwise, so decrement.
+                 */
+                else if(FP_CONTAINS_BOTTOM(seg2.y,point->y,seg1.y) && side<0)
+                {
+#ifdef PGIS_DEBUG
+                        lwnotice("decrementing winding number.");
+#endif
+                        --wn;
+                }
+        }
+
+#ifdef PGIS_DEBUG
+        lwnotice("winding number %d", wn);
+#endif
+        if(wn == 0)
+        	return -1;
+        return 1;
 }
 
 /*
- * return 0 iff point outside polygon
+ * return 0 iff point outside polygon or on boundary
+ * return 1 iff point inside polygon
  */
-int point_in_polygon(LWPOLY *polygon, LWPOINT *point)
+int point_in_polygon(RTREE_NODE **root, int ringCount, LWPOINT *point)
 {
         int i;
-        POINTARRAY *ring;
         POINT2D pt;
 
 #ifdef PGIS_DEBUG_CALLS
-        lwnotice("point_in_polygon called.");
+        lwnotice("point_in_polygon called for %p %d %p.", root, ringCount, point);
 #endif
 
         getPoint2d_p(point->point, 0, &pt);
         /* assume bbox short-circuit has already been attempted */
         
-        ring = polygon->rings[0];
-        if(point_in_ring(ring, &pt) == 0) 
+        if(point_in_ring(root[0], &pt) != 1) 
         {
 #ifdef PGIS_DEBUG
                 lwnotice("point_in_polygon: outside exterior ring.");
@@ -1145,10 +1326,9 @@ int point_in_polygon(LWPOLY *polygon, LWPOINT *point)
                 return 0;
         }
 
-        for(i=1; i<polygon->nrings; i++)
+        for(i=1; i<ringCount; i++)
         {
-                ring = polygon->rings[i];
-                if(point_in_ring(ring, &pt) != 0) 
+                if(point_in_ring(root[i], &pt) != -1)
                 {
 #ifdef PGIS_DEBUG
                         lwnotice("point_in_polygon: within hole %d.", i);
@@ -1159,9 +1339,136 @@ int point_in_polygon(LWPOLY *polygon, LWPOINT *point)
         return 1;
 }
 
+/*
+ * return 0 iff point outside polygon or on boundary
+ * return 1 iff point inside polygon
+ */
+int point_in_polygon_deprecated(LWPOLY *polygon, LWPOINT *point)
+{
+        int i;
+        POINTARRAY *ring;
+        POINT2D pt;
+
+#ifdef PGIS_DEBUG_CALLS
+        lwnotice("point_in_polygon_deprecated called.");
+#endif
+
+        getPoint2d_p(point->point, 0, &pt);
+        /* assume bbox short-circuit has already been attempted */
+        
+        ring = polygon->rings[0];
+        //root = createTree(ring);
+        //if(point_in_ring(root, &pt) != 1) 
+        if(point_in_ring_deprecated(polygon->rings[0], &pt) != 1)
+        {
+#ifdef PGIS_DEBUG
+                lwnotice("point_in_polygon: outside exterior ring.");
+#endif
+                return 0;
+        }
+
+        for(i=1; i<polygon->nrings; i++)
+        {
+                ring = polygon->rings[i];
+                //root = createTree(ring);
+                //if(point_in_ring(root, &pt) != -1) 
+                if(point_in_ring_deprecated(polygon->rings[i], &pt) != -1)
+                {
+#ifdef PGIS_DEBUG
+                        lwnotice("point_in_polygon: within hole %d.", i);
+#endif
+                        return 0;
+                }
+        }
+        return 1;
+}
+
+/*
+ * return 0 iff point inside polygon or on boundary
+ * return 1 iff point outside polygon
+ */
+int point_outside_polygon(RTREE_NODE **root, int ringCount, LWPOINT *point)
+{
+        int i;
+        POINT2D pt;
+
+#ifdef PGIS_DEBUG_CALLS
+        lwnotice("point_outside_polygon called.");
+#endif
+
+        getPoint2d_p(point->point, 0, &pt);
+        /* assume bbox short-circuit has already been attempted */
+        
+        if(point_in_ring(root[0], &pt) == -1)
+        {
+#ifdef PGIS_DEBUG
+                lwnotice("point_outside_polygon: outside exterior ring.");
+#endif
+                return 1;
+        }
+
+        for(i=1; i<ringCount; i++)
+        {
+                if(point_in_ring(root[i], &pt) == 1)
+                {
+#ifdef PGIS_DEBUG
+                        lwnotice("point_outside_polygon: within hole %d.", i);
+#endif
+                        return 1;
+                }
+        }
+        return 0;
+}
+
+/*
+ * return 0 iff point inside polygon or on boundary
+ * return 1 iff point outside polygon
+ */
+int point_outside_polygon_deprecated(LWPOLY *polygon, LWPOINT *point)
+{
+        int i;
+        POINTARRAY *ring;
+        POINT2D pt;
+
+#ifdef PGIS_DEBUG_CALLS
+        lwnotice("point_outside_polygon_deprecated called.");
+#endif
+
+        getPoint2d_p(point->point, 0, &pt);
+        /* assume bbox short-circuit has already been attempted */
+        
+        ring = polygon->rings[0];
+        //root = createTree(ring);
+        //if(point_in_ring(root, &pt) == -1) 
+        if(point_in_ring_deprecated(ring, &pt) == -1)
+        {
+#ifdef PGIS_DEBUG
+                lwnotice("point_outside_polygon_deprecated: outside exterior ring.");
+#endif
+                return 1;
+        }
+
+        for(i=1; i<polygon->nrings; i++)
+        {
+                ring = polygon->rings[i];
+                //root = createTree(ring);
+                //if(point_in_ring(root, &pt) == 1) 
+                if(point_in_ring_deprecated(ring, &pt) == 1)
+                {
+#ifdef PGIS_DEBUG
+                        lwnotice("point_outside_polygon_deprecated: within hole %d.", i);
+#endif
+                        return 1;
+                }
+        }
+        return 0;
+}
+
+
 /*
  * return 0 iff point is outside every polygon
  */
+/* Not yet functional.
 int point_in_multipolygon(LWMPOLY *mpolygon, LWPOINT *point)
 {
         int i;
@@ -1176,6 +1483,7 @@ int point_in_multipolygon(LWMPOLY *mpolygon, LWPOINT *point)
         }
         return 0;
 }
+*/
 
 
 /*******************************************************************************
