@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwgeom_functions_basic.c 2713 2007-10-29 13:44:29Z mcayland $
+ * $Id: lwgeom_functions_basic.c 2761 2008-04-10 06:40:33Z mcayland $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -1360,7 +1360,7 @@ Datum LWGEOM_force_2d(PG_FUNCTION_ARGS)
 	if ( lwgeom_ndims(geom->type) == 2 ) PG_RETURN_POINTER(geom);
 
 	/* allocate a larger for safety and simplicity */
-	srl = lwalloc(geom->size);
+	srl = lwalloc(VARSIZE(geom));
 
 	lwgeom_force2d_recursive(SERIALIZED_FORM(geom),
 		srl, &size);
@@ -1388,10 +1388,10 @@ Datum LWGEOM_force_3dz(PG_FUNCTION_ARGS)
 	if ( olddims == 3 && TYPE_HASZ(geom->type) ) PG_RETURN_POINTER(geom);
 
 	if ( olddims > 3 ) {
-		srl = lwalloc(geom->size);
+		srl = lwalloc(VARSIZE(geom));
 	} else {
 		/* allocate double as memory a larger for safety */
-		srl = lwalloc(geom->size*1.5);
+		srl = lwalloc(VARSIZE(geom)*1.5);
 	}
 
 	lwgeom_force3dz_recursive(SERIALIZED_FORM(geom),
@@ -1420,10 +1420,10 @@ Datum LWGEOM_force_3dm(PG_FUNCTION_ARGS)
 	if ( olddims == 3 && TYPE_HASM(geom->type) ) PG_RETURN_POINTER(geom);
 
 	if ( olddims > 3 ) {
-		size = geom->size;
+		size = VARSIZE(geom);
 	} else {
 		/* allocate double as memory a larger for safety */
-		size = geom->size * 2;
+		size = VARSIZE(geom) * 2;
 	}
 	srl = lwalloc(size);
 
@@ -1462,7 +1462,7 @@ Datum LWGEOM_force_4d(PG_FUNCTION_ARGS)
 	if ( olddims == 4 ) PG_RETURN_POINTER(geom);
 
 	/* allocate double as memory a larger for safety  */
-	srl = lwalloc(geom->size*2);
+	srl = lwalloc(VARSIZE(geom)*2);
 
 	lwgeom_force4d_recursive(SERIALIZED_FORM(geom),
 		srl, &size);
@@ -1879,7 +1879,7 @@ Datum LWGEOM_accum(PG_FUNCTION_ARGS)
 		elog(NOTICE, "geom_accum: NULL array");
 #endif
 	} else {
-		array = (ArrayType *) PG_DETOAST_DATUM_COPY(datum);
+		array = DatumGetArrayTypePCopy(datum);
 		/*array = PG_GETARG_ARRAYTYPE_P(0); */
 		nelems = ArrayGetNItems(ARR_NDIM(array), ARR_DIMS(array));
 #ifdef PGIS_DEBUG
@@ -1911,7 +1911,7 @@ Datum LWGEOM_accum(PG_FUNCTION_ARGS)
 	 */
 	++nelems;
 	if ( nelems == 1 || ! array ) {
-		nbytes = ARR_OVERHEAD_NONULLS(1)+INTALIGN(geom->size);
+		nbytes = ARR_OVERHEAD_NONULLS(1)+INTALIGN(VARSIZE(geom));
 #ifdef PGIS_DEBUG
 		elog(NOTICE, "geom_accum: adding %p (nelems=%d; nbytes=%d)",
 			(void*)geom, nelems, nbytes);
@@ -1934,16 +1934,16 @@ Datum LWGEOM_accum(PG_FUNCTION_ARGS)
 #endif
 		memcpy(ARR_DIMS(result), &nelems, sizeof(int));
 		memcpy(ARR_LBOUND(result), &lbs, sizeof(int));
-		memcpy(ARR_DATA_PTR(result), geom, geom->size);
+		memcpy(ARR_DATA_PTR(result), geom, VARSIZE(geom));
 #ifdef PGIS_DEBUG
-		elog(NOTICE, " %d bytes memcopied", geom->size);
+		elog(NOTICE, " %d bytes memcopied", VARSIZE(geom));
 #endif
 
 	} else {
 		oldsize = VARSIZE(array);
-		nbytes = oldsize + INTALIGN(geom->size);
+		nbytes = oldsize + INTALIGN(VARSIZE(geom));
 #ifdef PGIS_DEBUG
-		elog(NOTICE, "geom_accum: old array size: %d, adding %ld bytes (nelems=%d; nbytes=%u)", array->size, INTALIGN(geom->size), nelems, nbytes);
+		elog(NOTICE, "geom_accum: old array size: %d, adding %ld bytes (nelems=%d; nbytes=%u)", ARR_SIZE(array), INTALIGN(VARSIZE(geom)), nelems, nbytes);
 #endif
 		result = (ArrayType *) lwrealloc(array, nbytes);
 		if ( ! result )
@@ -1967,7 +1967,7 @@ Datum LWGEOM_accum(PG_FUNCTION_ARGS)
 		elog(NOTICE, " writing next element starting @ %p",
 			(void*)(result+oldsize));
 #endif
-		memcpy((uchar *)result+oldsize, geom, geom->size);
+		memcpy((uchar *)result+oldsize, geom, VARSIZE(geom));
 	}
 
 #ifdef PGIS_DEBUG
@@ -2021,11 +2021,11 @@ Datum LWGEOM_collect_garray(PG_FUNCTION_ARGS)
 	}
 
 	/* Get actual ArrayType */
-	array = (ArrayType *) PG_DETOAST_DATUM(datum);
+	array = DatumGetArrayTypeP(datum);
 
 #ifdef PGIS_DEBUG
 	elog(NOTICE, " array is %d-bytes in size, %ld w/out header",
-		array->size, array->size-ARR_OVERHEAD_NONULLS(ARR_NDIM(array)));
+		ARR_SIZE(array), ARR_SIZE(array)-ARR_OVERHEAD_NONULLS(ARR_NDIM(array)));
 #endif
 
 
@@ -2055,7 +2055,7 @@ Datum LWGEOM_collect_garray(PG_FUNCTION_ARGS)
 		PG_LWGEOM *geom = (PG_LWGEOM *)(ARR_DATA_PTR(array)+offset);
 		unsigned int intype = TYPE_GETTYPE(geom->type);
 
-		offset += INTALIGN(geom->size);
+		offset += INTALIGN(VARSIZE(geom));
 
 		lwgeoms[i] = lwgeom_deserialize(SERIALIZED_FORM(geom));
 #ifdef PGIS_DEBUG
@@ -2217,7 +2217,7 @@ Datum LWGEOM_makeline_garray(PG_FUNCTION_ARGS)
 	}
 
 	/* Get actual ArrayType */
-	array = (ArrayType *) PG_DETOAST_DATUM(datum);
+	array = DatumGetArrayTypeP(datum);
 
 #ifdef PGIS_DEBUG
 	elog(NOTICE, "LWGEOM_makeline_garray: array detoasted");
@@ -2250,7 +2250,7 @@ Datum LWGEOM_makeline_garray(PG_FUNCTION_ARGS)
 	for (i=0; i<nelems; i++)
 	{
 		PG_LWGEOM *geom = (PG_LWGEOM *)(ARR_DATA_PTR(array)+offset);
-		offset += INTALIGN(geom->size);
+		offset += INTALIGN(VARSIZE(geom));
 
 		if ( TYPE_GETTYPE(geom->type) != POINTTYPE ) continue;
 
@@ -2377,14 +2377,14 @@ Datum LWGEOM_makepoly(PG_FUNCTION_ARGS)
 	/* Get input holes if any */
 	if ( PG_NARGS() > 1 )
 	{
-		array = (ArrayType *) PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+		array = PG_GETARG_ARRAYTYPE_P(1);
 		nholes = ArrayGetNItems(ARR_NDIM(array), ARR_DIMS(array));
 		holes = lwalloc(sizeof(LWLINE *)*nholes);
 		for (i=0; i<nholes; i++)
 		{
 			PG_LWGEOM *g = (PG_LWGEOM *)(ARR_DATA_PTR(array)+offset);
 			LWLINE *hole;
-			offset += INTALIGN(g->size);
+			offset += INTALIGN(VARSIZE(g));
 			if ( TYPE_GETTYPE(g->type) != LINETYPE ) {
 				lwerror("Hole %d is not a line", i);
 			}
@@ -2767,14 +2767,14 @@ Datum LWGEOM_same(PG_FUNCTION_ARGS)
 	if ( TYPE_GETTYPE(g1->type) != TYPE_GETTYPE(g2->type) )
 	{
 		PG_FREE_IF_COPY(g1, 0);
-		PG_FREE_IF_COPY(g2, 0);
+		PG_FREE_IF_COPY(g2, 1);
 		PG_RETURN_BOOL(FALSE); /* different types */
 	}
 
 	if ( TYPE_GETZM(g1->type) != TYPE_GETZM(g2->type) )
 	{
 		PG_FREE_IF_COPY(g1, 0);
-		PG_FREE_IF_COPY(g2, 0);
+		PG_FREE_IF_COPY(g2, 1);
 		PG_RETURN_BOOL(FALSE); /* different dimensions */
 	}
 
