@@ -3,6 +3,7 @@
 #include "utils/builtins.h"
 #include "fmgr.h"
 
+
 #include "lwgeom_pg.h"
 #include "liblwgeom.h"
 #include "profile.h"
@@ -10,6 +11,13 @@
 #include "geos_c.h"
 #include "lwgeom_rtree.h"
 
+#include <string.h>
+
+
+#ifdef PROFILE
+#warning PROFILE enabled!
+#endif
+
 /*
  * Define this to have have many notices printed
  * during postgis->geos and geos->postgis conversions
@@ -52,6 +60,7 @@ Datum isvalid(PG_FUNCTION_ARGS);
 Datum buffer(PG_FUNCTION_ARGS);
 Datum intersection(PG_FUNCTION_ARGS);
 Datum convexhull(PG_FUNCTION_ARGS);
+Datum topologypreservesimplify(PG_FUNCTION_ARGS);
 Datum difference(PG_FUNCTION_ARGS);
 Datum boundary(PG_FUNCTION_ARGS);
 Datum symdifference(PG_FUNCTION_ARGS);
@@ -124,7 +133,7 @@ Datum unite_garray(PG_FUNCTION_ARGS)
 	/* Null array, null geometry (should be empty?) */
 	if ( (Pointer *)datum == NULL ) PG_RETURN_NULL();
 
-	array = (ArrayType *) PG_DETOAST_DATUM(datum);
+	array = DatumGetArrayTypeP(datum);
 
 	nelems = ArrayGetNItems(ARR_NDIM(array), ARR_DIMS(array));
 
@@ -240,7 +249,7 @@ Datum unite_garray(PG_FUNCTION_ARGS)
 	/* Null array, null geometry (should be empty?) */
 	if ( (Pointer *)datum == NULL ) PG_RETURN_NULL();
 
-	array = (ArrayType *) PG_DETOAST_DATUM(datum);
+	array = DatumGetArrayTypeP(datum);
 
 	nelems = ArrayGetNItems(ARR_NDIM(array), ARR_DIMS(array));
 
@@ -266,7 +275,7 @@ Datum unite_garray(PG_FUNCTION_ARGS)
 	while (!result) 
 	{
 		PG_LWGEOM *geom = (PG_LWGEOM *)(ARR_DATA_PTR(array)+offset);
-		offset += INTALIGN(geom->size);
+		offset += INTALIGN(VARSIZE(geom));
 
 		/* Check is3d flag */
 		if ( TYPE_HASZ(geom->type) ) is3d = 1;
@@ -743,6 +752,86 @@ Datum convexhull(PG_FUNCTION_ARGS)
 
 }
 
+#if GEOS_VERNUM >= 30 
+
+PG_FUNCTION_INFO_V1(topologypreservesimplify);
+Datum topologypreservesimplify(PG_FUNCTION_ARGS)
+{
+	PG_LWGEOM	*geom1;
+	double	tolerance;
+	GEOSGeom g1,g3;
+	PG_LWGEOM *result;
+
+#ifdef PROFILE
+	profstart(PROF_QRUN);
+#endif
+
+	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	tolerance = PG_GETARG_FLOAT8(1);
+
+	initGEOS(lwnotice, lwnotice);
+
+#ifdef PROFILE
+	profstart(PROF_P2G1);
+#endif
+	g1 = POSTGIS2GEOS(geom1);
+#ifdef PROFILE
+	profstop(PROF_P2G1);
+#endif
+
+#ifdef PROFILE
+	profstart(PROF_GRUN);
+#endif
+	g3 = GEOSTopologyPreserveSimplify(g1,tolerance);
+#ifdef PROFILE
+	profstop(PROF_GRUN);
+#endif
+
+	if (g3 == NULL)
+	{
+		elog(ERROR,"GEOS topologypreservesimplify() threw an error!");
+		GEOSGeom_destroy(g1);
+		PG_RETURN_NULL(); /* never get here */
+	}
+
+
+#ifdef PGIS_DEBUG
+	elog(NOTICE,"result: %s", GEOSGeomToWKT(g3));
+#endif
+
+	GEOSSetSRID(g3, pglwgeom_getSRID(geom1));
+
+#ifdef PROFILE
+	profstart(PROF_G2P);
+#endif
+	result = GEOS2POSTGIS(g3, TYPE_HASZ(geom1->type));
+#ifdef PROFILE
+	profstop(PROF_G2P);
+#endif
+	if (result == NULL)
+	{
+		GEOSGeom_destroy(g1);
+		GEOSGeom_destroy(g3);
+		elog(ERROR,"GEOS topologypreservesimplify() threw an error (result postgis geometry formation)!");
+		PG_RETURN_NULL(); /* never get here */
+	}
+	GEOSGeom_destroy(g1);
+	GEOSGeom_destroy(g3);
+
+
+	/* compressType(result); */
+
+#ifdef PROFILE
+	profstop(PROF_QRUN);
+	profreport("geos",geom1, NULL, result);
+#endif
+
+	PG_FREE_IF_COPY(geom1, 0);
+
+	PG_RETURN_POINTER(result);
+}
+#endif
+
 PG_FUNCTION_INFO_V1(buffer);
 Datum buffer(PG_FUNCTION_ARGS)
 {
@@ -1347,6 +1436,7 @@ int point_outside_polygon(RTREE_NODE **root, int ringCount, LWPOINT *point);
 int point_outside_polygon_deprecated(LWPOLY *polygon, LWPOINT *point);
 int point_in_multipolygon(LWMPOLY *mpolygon, LWPOINT *point);
 
+
 PG_FUNCTION_INFO_V1(contains);
 Datum contains(PG_FUNCTION_ARGS)
 {
@@ -1499,7 +1589,7 @@ Datum contains(PG_FUNCTION_ARGS)
 
 #ifdef PROFILE
 	profstop(PROF_QRUN);
-	profreport("geos",geom1, geom2, NULL);
+	profreport("cont",geom1, geom2, NULL);
 #endif
 
 	PG_FREE_IF_COPY(geom1, 0);
@@ -1508,6 +1598,7 @@ Datum contains(PG_FUNCTION_ARGS)
 	PG_RETURN_BOOL(result);
 }
 
+
 /*
  * Described at 
  * http://lin-ear-th-inking.blogspot.com/2007/06/subtleties-of-ogc-covers-spatial.html
@@ -3271,7 +3362,7 @@ Datum polygonize_garray(PG_FUNCTION_ARGS)
 	/* Null array, null geometry (should be empty?) */
 	if ( (Pointer *)datum == NULL ) PG_RETURN_NULL();
 
-	array = (ArrayType *) PG_DETOAST_DATUM(datum);
+	array = DatumGetArrayTypeP(datum);
 
 	nelems = ArrayGetNItems(ARR_NDIM(array), ARR_DIMS(array));
 
@@ -3575,3 +3666,425 @@ Datum LWGEOM_buildarea(PG_FUNCTION_ARGS)
 	PG_RETURN_POINTER(result);
 
 }
+
+
+Datum containsPrepared(PG_FUNCTION_ARGS);
+Datum containsProperlyPrepared(PG_FUNCTION_ARGS);
+Datum coversPrepared(PG_FUNCTION_ARGS);
+Datum intersectsPrepared(PG_FUNCTION_ARGS);
+
+/*
+ * GEOS prepared geometry is only available from GEOS 3.1 onwards
+ */
+#if GEOS_VERNUM >= 31
+#define PREPARED_GEOM 1
+#endif
+
+
+#ifdef PREPARED_GEOM
+
+typedef struct
+{
+	int32					key;
+	GEOSPreparedGeometry * 	prepared_geom;
+} PREPARED_GEOM_CACHE;
+
+
+/* 
+ * get cached prepared geometry for geom1
+ * only geom1 is potentially prepared as only
+ * the first arg of the prepared predicates CAN be prepared
+ * briefly, the following does:
+ *
+ * get cache
+ * if cache not exist
+ *   create cache 
+ *   key into cache
+ *
+ * else if key matches cached key
+ *    if cached prepared not exist
+ *        geom1 prepared into cache
+ *    
+ * else
+ *   key into cache
+ *   clear prepared cache
+ */
+PREPARED_GEOM_CACHE *
+get_prepared_geometry_cache( 
+	PREPARED_GEOM_CACHE * 	cache, 
+	uchar * 				serialized_geom,
+	int32					key )
+{
+	GEOSGeom g;
+
+	if ( !cache )
+	{
+		#ifdef PGIS_DEBUG
+		lwnotice( "get_prepared_geometry_cache: creating cache: %x", cache);
+		#endif
+		
+		cache = lwalloc( sizeof( PREPARED_GEOM_CACHE ));
+		
+		cache->prepared_geom = 0;
+		cache->key = key;
+	}
+	else if ( cache->key == key )
+	{
+		if ( !cache->prepared_geom )
+		{
+			#ifdef PGIS_DEBUG
+			lwnotice("get_prepared_geometry_cache: preparing obj");
+			#endif
+			
+			g = POSTGIS2GEOS( serialized_geom);
+			cache->prepared_geom = GEOSPrepare( g);
+		}
+		else
+		{
+			#ifdef PGIS_DEBUG
+			lwnotice("get_prepared_geometry_cache: prepared obj in cache");
+			#endif
+		}
+	}
+	else
+	{
+		#ifdef PGIS_DEBUG
+		lwnotice("get_prepared_geometry_cache: obj NOT in cache");
+		#endif
+
+		GEOSPreparedGeom_destroy( cache->prepared_geom);
+
+		cache->prepared_geom = 0;
+		cache->key = key;
+	}
+	
+	return cache;
+}
+
+#endif /* PREPARED_GEOM */
+
+
+PG_FUNCTION_INFO_V1(containsPrepared);
+Datum containsPrepared(PG_FUNCTION_ARGS)
+{
+#ifndef PREPARED_GEOM
+	elog(ERROR,"Not implemented in this version!");
+	PG_RETURN_NULL(); /* never get here */
+
+#else
+	PG_LWGEOM *				geom1;
+	PG_LWGEOM *				geom2;
+	GEOSGeom 				g1, g2;
+	GEOSPreparedGeometry * 	pg;
+	bool 					result;
+	BOX2DFLOAT4 			box1, box2;
+	PREPARED_GEOM_CACHE *	prep_cache;
+	MemoryContext 			old_context;
+	int32					surrogate_key;
+
+	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	
+	surrogate_key = PG_GETARG_INT32(2);
+	
+	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
+
+	/*
+	 * short-circuit: if geom2 bounding box is not completely inside
+	 * geom1 bounding box we can prematurely return FALSE.
+	 * Do the test IFF BOUNDING BOX AVAILABLE.
+	 */
+	if ( getbox2d_p(SERIALIZED_FORM(geom1), &box1) &&
+		getbox2d_p(SERIALIZED_FORM(geom2), &box2) )
+	{
+		if ( box2.xmin < box1.xmin ) PG_RETURN_BOOL(FALSE);
+		if ( box2.xmax > box1.xmax ) PG_RETURN_BOOL(FALSE);
+		if ( box2.ymin < box1.ymin ) PG_RETURN_BOOL(FALSE);
+		if ( box2.ymax > box1.ymax ) PG_RETURN_BOOL(FALSE);
+	}
+	
+	old_context = MemoryContextSwitchTo(fcinfo->flinfo->fn_mcxt);
+	prep_cache =  fcinfo->flinfo->fn_extra;
+
+	prep_cache = get_prepared_geometry_cache( prep_cache, geom1, surrogate_key);
+
+	fcinfo->flinfo->fn_extra = prep_cache;
+	MemoryContextSwitchTo(old_context);
+
+	initGEOS(lwnotice, lwnotice);
+
+	g2 = POSTGIS2GEOS(geom2);
+
+	if ( !prep_cache || !prep_cache->prepared_geom )
+	{
+		g1 = POSTGIS2GEOS(geom1);
+
+		result = GEOSContains( g1, g2);
+		
+		GEOSGeom_destroy(g1);
+	}
+	else
+	{
+		pg = prep_cache->prepared_geom;
+
+		result = GEOSPreparedContains( pg, g2);	
+	}
+	GEOSGeom_destroy(g2);
+
+	if (result == 2)
+	{
+		elog(ERROR,"GEOS contains() threw an error!");
+		PG_RETURN_NULL(); /* never get here */
+	}
+
+	PG_FREE_IF_COPY(geom1, 0);
+	PG_FREE_IF_COPY(geom2, 1);
+
+	PG_RETURN_BOOL(result);
+#endif /* PREPARED_GEOM */
+}
+
+PG_FUNCTION_INFO_V1(containsProperlyPrepared);
+Datum containsProperlyPrepared(PG_FUNCTION_ARGS)
+{
+#ifndef PREPARED_GEOM
+	elog(ERROR,"Not implemented in this version!");
+	PG_RETURN_NULL(); /* never get here */
+
+#else
+	PG_LWGEOM *				geom1;
+	PG_LWGEOM *				geom2;
+	GEOSGeom 				g1, g2;
+	GEOSPreparedGeometry * 	pg;
+	bool 					result;
+	BOX2DFLOAT4 			box1, box2;
+	PREPARED_GEOM_CACHE *	prep_cache;
+	MemoryContext 			old_context;
+	int32					surrogate_key;
+
+	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	
+	surrogate_key = PG_GETARG_INT32(2);
+	
+	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
+
+	/*
+	 * short-circuit: if geom2 bounding box is not completely inside
+	 * geom1 bounding box we can prematurely return FALSE.
+	 * Do the test IFF BOUNDING BOX AVAILABLE.
+	 */
+	if ( getbox2d_p(SERIALIZED_FORM(geom1), &box1) &&
+		getbox2d_p(SERIALIZED_FORM(geom2), &box2) )
+	{
+		if ( box2.xmin < box1.xmin ) PG_RETURN_BOOL(FALSE);
+		if ( box2.xmax > box1.xmax ) PG_RETURN_BOOL(FALSE);
+		if ( box2.ymin < box1.ymin ) PG_RETURN_BOOL(FALSE);
+		if ( box2.ymax > box1.ymax ) PG_RETURN_BOOL(FALSE);
+	}
+	
+	old_context = MemoryContextSwitchTo(fcinfo->flinfo->fn_mcxt);
+	prep_cache =  fcinfo->flinfo->fn_extra;
+	
+	prep_cache = get_prepared_geometry_cache( prep_cache, geom1, surrogate_key);
+	
+	fcinfo->flinfo->fn_extra = prep_cache;
+	MemoryContextSwitchTo( old_context);
+
+	initGEOS(lwnotice, lwnotice);
+		
+	g2 = POSTGIS2GEOS(geom2);
+
+	if ( !prep_cache || !prep_cache->prepared_geom )
+	{
+		g1 = POSTGIS2GEOS(geom1);
+		
+		result = GEOSRelatePattern( g1, g2, "T**FF*FF*");
+		
+		GEOSGeom_destroy(g1);
+	}
+	else
+	{
+		pg = prep_cache->prepared_geom;
+
+		result = GEOSPreparedContainsProperly( pg, g2);	
+	}
+	GEOSGeom_destroy(g2);
+
+	if (result == 2)
+	{
+		elog(ERROR,"GEOS contains() threw an error!");
+		PG_RETURN_NULL(); /* never get here */
+	}
+
+	PG_FREE_IF_COPY(geom1, 0);
+	PG_FREE_IF_COPY(geom2, 1);
+
+	PG_RETURN_BOOL(result);
+#endif /* PREPARED_GEOM */
+}
+
+PG_FUNCTION_INFO_V1(coversPrepared);
+Datum coversPrepared(PG_FUNCTION_ARGS)
+{
+#ifndef PREPARED_GEOM
+	elog(ERROR,"Not implemented in this version!");
+	PG_RETURN_NULL(); /* never get here */
+
+#else
+	PG_LWGEOM *				geom1;
+	PG_LWGEOM *				geom2;
+	GEOSGeom 				g1, g2;
+	GEOSPreparedGeometry * 	pg;
+	bool 					result;
+	BOX2DFLOAT4 			box1, box2;
+	PREPARED_GEOM_CACHE *	prep_cache;
+	MemoryContext 			old_context;
+	int32					surrogate_key;
+
+	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	
+	surrogate_key = PG_GETARG_INT32(2);
+	
+	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
+
+	/*
+	 * short-circuit: if geom2 bounding box is not completely inside
+	 * geom1 bounding box we can prematurely return FALSE.
+	 * Do the test IFF BOUNDING BOX AVAILABLE.
+	 */
+	if ( getbox2d_p(SERIALIZED_FORM(geom1), &box1) &&
+		getbox2d_p(SERIALIZED_FORM(geom2), &box2) )
+	{
+		if ( box2.xmin < box1.xmin ) PG_RETURN_BOOL(FALSE);
+		if ( box2.xmax > box1.xmax ) PG_RETURN_BOOL(FALSE);
+		if ( box2.ymin < box1.ymin ) PG_RETURN_BOOL(FALSE);
+		if ( box2.ymax > box1.ymax ) PG_RETURN_BOOL(FALSE);
+	}
+	
+	old_context = MemoryContextSwitchTo(fcinfo->flinfo->fn_mcxt);
+	prep_cache =  fcinfo->flinfo->fn_extra;
+
+	prep_cache = get_prepared_geometry_cache( prep_cache, geom1, surrogate_key);
+
+	fcinfo->flinfo->fn_extra = prep_cache;
+	MemoryContextSwitchTo(old_context);
+
+	initGEOS(lwnotice, lwnotice);
+
+	g2 = POSTGIS2GEOS(geom2);
+
+	if ( !prep_cache || !prep_cache->prepared_geom )
+	{
+		g1 = POSTGIS2GEOS(geom1);
+
+		result = GEOSRelatePattern( g1, g2, "******FF*");
+		
+		GEOSGeom_destroy(g1);
+	}
+	else
+	{
+		pg = prep_cache->prepared_geom;
+
+		result = GEOSPreparedCovers( pg, g2);	
+	}
+	GEOSGeom_destroy(g2);
+
+	if (result == 2)
+	{
+		elog(ERROR,"GEOS contains() threw an error!");
+		PG_RETURN_NULL(); /* never get here */
+	}
+
+	PG_FREE_IF_COPY(geom1, 0);
+	PG_FREE_IF_COPY(geom2, 1);
+
+	PG_RETURN_BOOL(result);
+#endif /* PREPARED_GEOM */
+}
+
+PG_FUNCTION_INFO_V1(intersectsPrepared);
+Datum intersectsPrepared(PG_FUNCTION_ARGS)
+{
+#ifndef PREPARED_GEOM
+	elog(ERROR,"Not implemented in this version!");
+	PG_RETURN_NULL(); /* never get here */
+
+#else
+	PG_LWGEOM *				geom1;
+	PG_LWGEOM *				geom2;
+	GEOSGeom 				g1, g2;
+	GEOSPreparedGeometry * 	pg;
+	bool 					result;
+	BOX2DFLOAT4 			box1, box2;
+	PREPARED_GEOM_CACHE *	prep_cache;
+	MemoryContext 			old_context;
+	int32					surrogate_key;
+
+	geom1 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(0));
+	geom2 = (PG_LWGEOM *)  PG_DETOAST_DATUM(PG_GETARG_DATUM(1));
+	
+	surrogate_key = PG_GETARG_INT32(2);
+	
+	errorIfGeometryCollection(geom1,geom2);
+	errorIfSRIDMismatch(pglwgeom_getSRID(geom1), pglwgeom_getSRID(geom2));
+
+	/*
+	 * short-circuit 1: if geom2 bounding box does not overlap
+	 * geom1 bounding box we can prematurely return FALSE.
+	 * Do the test IFF BOUNDING BOX AVAILABLE.
+	 */
+	if ( getbox2d_p(SERIALIZED_FORM(geom1), &box1) &&
+		getbox2d_p(SERIALIZED_FORM(geom2), &box2) )
+	{
+		if ( box2.xmax < box1.xmin ) PG_RETURN_BOOL(FALSE);
+		if ( box2.xmin > box1.xmax ) PG_RETURN_BOOL(FALSE);
+		if ( box2.ymax < box1.ymin ) PG_RETURN_BOOL(FALSE);
+		if ( box2.ymin > box2.ymax ) PG_RETURN_BOOL(FALSE);
+	}
+	
+	old_context = MemoryContextSwitchTo(fcinfo->flinfo->fn_mcxt);
+	prep_cache =  fcinfo->flinfo->fn_extra;
+
+	prep_cache = get_prepared_geometry_cache( prep_cache, geom1, surrogate_key);
+
+	fcinfo->flinfo->fn_extra = prep_cache;
+	MemoryContextSwitchTo(old_context);
+
+	initGEOS(lwnotice, lwnotice);
+
+	g2 = POSTGIS2GEOS(geom2);
+
+	if ( !prep_cache || !prep_cache->prepared_geom )
+	{
+		g1 = POSTGIS2GEOS(geom1);
+
+		result = GEOSIntersects( g1, g2);
+		
+		GEOSGeom_destroy(g1);
+	}
+	else
+	{
+		pg = prep_cache->prepared_geom;
+
+		result = GEOSPreparedIntersects( pg, g2);	
+	}
+	GEOSGeom_destroy(g2);
+
+	if (result == 2)
+	{
+		elog(ERROR,"GEOS contains() threw an error!");
+		PG_RETURN_NULL(); /* never get here */
+	}
+
+	PG_FREE_IF_COPY(geom1, 0);
+	PG_FREE_IF_COPY(geom2, 1);
+
+	PG_RETURN_BOOL(result);
+#endif /* PREPARED_GEOM */
+}
+
