@@ -116,7 +116,7 @@ void WRITE_DOUBLES(output_state* out,double* points, int cnt);
 #ifdef SHRINK_INTS
 void WRITE_INT4(output_state * out,int4 val);
 #endif
-void write_size(tuple* this,output_state* out);
+void empty_stack(tuple* this,output_state* out);
 void alloc_lwgeom(int srid);
 void write_point_2(tuple* this,output_state* out);
 void write_point_3(tuple* this,output_state* out);
@@ -144,20 +144,23 @@ void alloc_multisurface(void);
 void alloc_geomertycollection(void);
 void alloc_counter(void);
 void alloc_empty(void);
-uchar* make_lwgeom(void);
+SERIALIZED_LWGEOM* make_serialized_lwgeom(void);
 uchar strhex_readbyte(const char *in);
 uchar read_wkb_byte(const char **in);
 void read_wkb_bytes(const char **in, uchar* out, int cnt);
 int4 read_wkb_int(const char **in);
 double read_wkb_double(const char **in, int convert_from_int);
 void read_wkb_point(const char **b);
+void read_wkb_polygon(const char **b);
+void read_wkb_linestring(const char **b);
+void read_wkb_curve(const char **b);
+void read_wkb_ordinate_array(const char **b);
 void read_collection(const char **b, read_col_func f);
-void read_collection2(const char **b);
 void parse_wkb(const char **b);
 void alloc_wkb(const char *parser);
-uchar* parse_it(const char* geometry, allocator allocfunc, report_error errfunc);
-uchar* parse_lwg(const char* geometry, allocator allocfunc, report_error errfunc);
-uchar* parse_lwgi(const char* geometry, allocator allocfunc, report_error errfunc);
+SERIALIZED_LWGEOM* parse_it(const char* geometry, allocator allocfunc, report_error errfunc);
+SERIALIZED_LWGEOM* parse_lwg(const char* geometry, allocator allocfunc, report_error errfunc);
+SERIALIZED_LWGEOM* parse_lwgi(const char* geometry, allocator allocfunc, report_error errfunc);
 
 void
 set_srid(double d_srid)
@@ -269,6 +272,7 @@ pop(void)
 void
 popc(void)
 {
+
 	if ( the_geom.stack->uu.nn.num < minpoints){
 		error("geometry requires more points");
 	}
@@ -281,7 +285,8 @@ popc(void)
 		{
 			error("geometry contains non-closed rings");
 		}
-	}
+	}	
+
 	the_geom.stack = the_geom.stack->uu.nn.stack_next;
 }
 
@@ -368,14 +373,9 @@ WRITE_DOUBLES(output_state* out,double* points, int cnt)
 }
 
 void
-write_size(tuple* this,output_state* out)
+empty_stack(tuple *this,output_state* out)
 {
-
-#ifdef PGIS_DEBUG_CALLS
-        lwnotice("write_size");
-#endif
-
-	WRITE_INT4_REAL(out,the_geom.alloc_size);
+    /* Do nothing but provide an empty base for the geometry stack */
 }
 
 void
@@ -403,7 +403,8 @@ alloc_lwgeom(int srid)
 		the_geom.alloc_size+=4;
 	}
 
-	the_geom.stack = alloc_tuple(write_size,4);
+    /* Setup up an empty tuple as the stack base */
+    the_geom.stack = alloc_tuple(empty_stack, 0);
 }
 
 void
@@ -784,17 +785,23 @@ alloc_empty(void)
 	st->uu.nn.num=0;
 }
 
-uchar *
-make_lwgeom(void)
+SERIALIZED_LWGEOM *
+make_serialized_lwgeom(void)
 {
 
 #ifdef PGIS_DEBUG_CALLS
-        lwnotice("make_lwgeom");
+        lwnotice("make_serialized_lwgeom");
 #endif
 
+    SERIALIZED_LWGEOM *out_serialized_lwgeom;
 	uchar* out_c;
 	output_state out;
 	tuple* cur;
+
+    /* Allocate the SERIALIZED_LWGEOM structure */
+    out_serialized_lwgeom = (SERIALIZED_LWGEOM *)local_malloc(sizeof(SERIALIZED_LWGEOM));
+
+    /* Allocate the LWGEOM itself */
 	out_c = (uchar*)local_malloc(the_geom.alloc_size);
 	out.pos = out_c;
 	cur = the_geom.first ;
@@ -804,11 +811,11 @@ make_lwgeom(void)
 		cur=cur->next;
 	}
 
-	/* if ints shrink then we need to rewrite the size smaller */
-	out.pos = out_c;
-	write_size(NULL,&out);
+	/* Setup the SERIALIZED_LWGEOM structure */
+    out_serialized_lwgeom->lwgeom = out_c;
+    out_serialized_lwgeom->size = the_geom.alloc_size;
 
-	return out_c;
+	return out_serialized_lwgeom;
 }
 
 void
@@ -944,13 +951,85 @@ read_wkb_point(const char **b)
 		}
 	}
 
+
+	/* keep track of point */
+	if ( checkclosed ) {
+		if ( ! the_geom.stack->uu.nn.num )
+			first_point = p->uu.points;
+		last_point = p->uu.points;
+	}
+
 	inc_num();
 	check_dims(the_geom.ndims);
 }
 
+void
+read_wkb_polygon(const char **b)
+{
+	/* Stack the number of ORDINATE_ARRAYs (rings) */
+	int4 cnt=read_wkb_int(b);
+	alloc_counter();
+
+	/* Read through each ORDINATE_ARRAY in turn */
+	while(cnt--){
+		if ( ferror_occured )	return;
+
+		/* Things to check for POLYGON ORDINATE_ARRAYs */
+		minpoints=3;
+		checkclosed=1;
+		isodd=-1;
+
+		read_wkb_ordinate_array(b);
+	}
+
+	pop();
+}
+
+void
+read_wkb_linestring(const char **b)
+{
+
+	/* Things to check for LINESTRING ORDINATE_ARRAYs */
+	minpoints=2;
+	checkclosed=0;
+	isodd=-1;
+
+	read_wkb_ordinate_array(b);
+}
+
+
+void
+read_wkb_curve(const char **b)
+{
+
+	/* Things to check for CURVE ORDINATE_ARRAYs */
+	minpoints=3;
+	checkclosed=0;
+	isodd=-1;
+
+	read_wkb_ordinate_array(b);
+}
+
+void
+read_wkb_ordinate_array(const char **b)
+{
+	/* Read through a WKB ordinate array */
+	int4 cnt=read_wkb_int(b);
+	alloc_counter();
+
+	while(cnt--){
+		if ( ferror_occured )	return;
+		read_wkb_point(b);
+	}
+
+	/* Perform a check of the ordinate array */
+	popc();
+}
+
 void
 read_collection(const char **b, read_col_func f)
 {
+	/* Read through a COLLECTION or an EWKB */
 	int4 cnt=read_wkb_int(b);
 	alloc_counter();
 
@@ -962,12 +1041,6 @@ read_collection(const char **b, read_col_func f)
 	pop();
 }
 
-void
-read_collection2(const char **b)
-{
-	read_collection(b, read_wkb_point);
-}
-
 void
 parse_wkb(const char **b)
 {
@@ -1041,15 +1114,15 @@ parse_wkb(const char **b)
 			break;
 
 		case	LINETYPE:
-			read_collection(b,read_wkb_point);
+			read_wkb_linestring(b);
 			break;
 
                 case    CURVETYPE:
-                        read_collection(b,read_wkb_point);
+                        read_wkb_curve(b);
                         break;
 
 		case	POLYGONTYPE:
-			read_collection(b,read_collection2);
+			read_wkb_polygon(b);
 			break;
 
                 case    COMPOUNDTYPE:
@@ -1076,12 +1149,12 @@ parse_wkb(const char **b)
 
 		case	LINETYPEI:
 			the_geom.from_lwgi=1;
-			read_collection(b,read_wkb_point);
+			read_wkb_linestring(b);
 			break;
 
 		case	POLYGONTYPEI:
 			the_geom.from_lwgi=1;
-			read_collection(b,read_collection2);
+			read_wkb_polygon(b);
 			break;
 
 		default:
@@ -1108,7 +1181,7 @@ alloc_wkb(const char *parser)
 /*
 	Parse a string and return a LW_GEOM
 */
-uchar *
+SERIALIZED_LWGEOM *
 parse_it(const char *geometry, allocator allocfunc, report_error errfunc)
 {
 
@@ -1130,17 +1203,17 @@ parse_it(const char *geometry, allocator allocfunc, report_error errfunc)
 	if (ferror_occured)
 		return NULL;
 
-	return make_lwgeom();
+	return make_serialized_lwgeom();
 }
 
-uchar *
+SERIALIZED_LWGEOM *
 parse_lwg(const char* geometry,allocator allocfunc,report_error errfunc)
 {
 	the_geom.lwgi=0;
 	return parse_it(geometry,allocfunc,errfunc);
 }
 
-uchar *
+SERIALIZED_LWGEOM *
 parse_lwgi(const char* geometry,allocator allocfunc,report_error errfunc)
 {
 	the_geom.lwgi=1;
