@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: lwline.c 2532 2006-12-01 22:16:44Z mleslie $
+ * $Id: lwline.c 2719 2007-11-27 22:39:06Z mcayland $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -98,7 +98,7 @@ lwline_deserialize(uchar *serialized_form)
 	if ( lwgeom_hasSRID(type))
 	{
 		/*lwnotice("line has srid"); */
-		result->SRID = get_int32(loc);
+		result->SRID = lw_get_int32(loc);
 		loc +=4; /* type + SRID */
 	}
 	else
@@ -109,7 +109,7 @@ lwline_deserialize(uchar *serialized_form)
 
 	/* we've read the type (1 byte) and SRID (4 bytes, if present) */
 
-	npoints = get_uint32(loc);
+	npoints = lw_get_uint32(loc);
 	/*lwnotice("line npoints = %d", npoints); */
 	loc +=4;
 	pa = pointArray_construct(loc, TYPE_HASZ(type), TYPE_HASM(type), npoints);
@@ -303,7 +303,7 @@ lwgeom_size_line(const uchar *serialized_line)
 	}
 
 	/* we've read the type (1 byte) and SRID (4 bytes, if present) */
-	npoints = get_uint32(loc);
+	npoints = lw_get_uint32(loc);
 	result += sizeof(uint32); /* npoints */
 
 	result += TYPE_NDIMS(type) * sizeof(double) * npoints;
@@ -334,6 +334,9 @@ lwline_compute_box2d_p(LWLINE *line, BOX2DFLOAT4 *box)
 LWLINE *
 lwline_clone(const LWLINE *g)
 {
+#ifdef PGIS_DEBUG_CALLS
+        lwnotice("lwline_clone called with %p", g);
+#endif
 	LWLINE *ret = lwalloc(sizeof(LWLINE));
 	memcpy(ret, g, sizeof(LWLINE));
 	if ( g->bbox ) ret->bbox = box2d_clone(g->bbox);
