@@ -1,5 +1,5 @@
 /**********************************************************************
- * $Id: measures.c 2369 2006-05-30 08:38:58Z strk $
+ * $Id: measures.c 2609 2007-05-28 15:34:04Z mleslie $
  *
  * PostGIS - Spatial Types for PostgreSQL
  * http://postgis.refractions.net
@@ -580,6 +580,16 @@ lwgeom_pointarray_length(POINTARRAY *pts)
 	return dist;
 }
 
+/*
+ * This should be rewritten to make use of the curve itself.
+ */
+double
+lwgeom_curvepolygon_area(LWCURVEPOLY *curvepoly)
+{
+        LWPOLY *poly = (LWPOLY *)lwgeom_segmentize((LWGEOM *)curvepoly, 32);
+        return lwgeom_polygon_area(poly);
+}
+
 /*
  * Find the area of the outer ring - sum (area of inner rings).
  * Could use a more numerically stable calculator...
